//###########################################################################//
//
//Project :   Solid Works  v 0.94 Beta  for Ti 89
//
//Author:  David Coz      
//
//Mail  :  coz.hubert@infonie.fr
//
// Misc. Functions :
// Triangle filler , sorting , waiting , key detection , Int To String ...
//
//Created 10/05/2002; 00:21:07
//
//############################################################################//



#define OPTIMIZE_ROM_CALLS     // Use ROM Call Optimization

#define STRICT_POINTERS


#include <tigcclib.h> 

 void *Vscreen1;

//Depth-sorting routine 

void sort(char *data,int *data3,int numData)
{

int count,index;
int  (*data4) = NULL;	 
int  (*data5) = NULL;	 

data4=calloc(numData,sizeof(*data4));
data5=calloc(numData,sizeof(*data4));

char *oneArray=calloc(numData,sizeof(*data));
char *zeroArray=calloc(numData,sizeof(*data));
	                  // 8 bit data in this example
       
        int numOnes;
        int numZeros;

        int mask = 1;

        for (count = 0; count < 8; count++)
        {
            numOnes = 0;
            numZeros = 0;

            for (index = 0; index < numData; index++)
            {
                if (data[index] & mask)
                {
                	
                		data4[numOnes]=data3[index];
                    oneArray[numOnes] = data[index];
                    numOnes++;
                }
                else
                {
                	
                		data5[numZeros]=data3[index];
                    zeroArray[numZeros] = data[index];
                    numZeros++;
                }
            }

            // put the 1s back in the original data array first

            memcpy(data, oneArray,sizeof(*data)*numOnes);
						memcpy(data3,data4, sizeof(*data4)*numOnes);

            // now put the 0s back
            memcpy(data+numOnes, zeroArray,sizeof(*data)*numZeros);
						memcpy(data3+numOnes,data5,sizeof(*data4)*numZeros);
            
            // mask out the next most significant bit next time through

            mask <<= 1;
        }
free(data4);
free(data5);	
	free(zeroArray);
	free(oneArray);
}

//wait a moment depending on delay

void wait(unsigned short int delay)
{
   OSFreeTimer(USER_TIMER);
  OSRegisterTimer(USER_TIMER,delay);
  while(!OSTimerExpired(USER_TIMER));
}




//Triangle Filling Routine
//Thanks to Thomas Nussbaumer for improving strongly this routine 
//*******************************************************************************

#define ASM_EXCHANGE(val1,val2) asm volatile ("exg %0,%1" : "=d" (val1),"=d" (val2) : "0" (val1),"1" (val2) : "memory","cc")
#define ASM_SWAP(val) ({\
    register unsigned long tmp = val;\
    asm volatile ("swap %0" : "=d" (tmp) : "0" (tmp));\
    tmp;})


/*===========================================================================*/
/* draws a horizontal line from (x1,y) to (x2,y)                             */
/*===========================================================================*/
//=============================================================================
// fills a horizontal line from x1 to x2 starting at address addr
//=============================================================================
static inline void FillSpan(short x1,short x2,unsigned char* addr) {
    static const unsigned short table1[16] = {0xffff,0x7fff,0x3fff,0x1fff,
                                              0x0fff,0x07ff,0x03ff,0x01ff,
                                              0x00ff,0x007f,0x003f,0x001f,
                                              0x000f,0x0007,0x0003,0x0001};
    static const unsigned short table2[16] = {0x0000,0x8000,0xc000,0xe000,
                                              0xf000,0xf800,0xfc00,0xfe00,
                                              0xff00,0xff80,0xffc0,0xffe0,
                                              0xfff0,0xfff8,0xfffc,0xfffe};
    short           dx;
    short           sx;
    unsigned short* p;

    if (x1>x2) {
        ASM_EXCHANGE(x1,x2);
    }
    if (x1<0){
       x1=0;
       if (x2<0)
       return;}
    if (x2>159) {
        x2=159;
        if (x1>159)
        return;}
        
    p  = (unsigned short*)(addr+((x1>>3)&0x1e));
    dx = x2-x1+1;
    sx = x1 & 0x000f;

    if (dx<16) {
        unsigned long val = (ASM_SWAP(table2[dx])) >> sx;

        *(unsigned long*)p |= val;

        return;
    }

    if (sx) {
        *p++ |= table1[sx];
        dx -= (16-sx);
    }
    while (dx >= 16) {
        *p++ |= 0xffff;
        dx-=16;
    }

    if (dx) *p |= table2[dx];

}

static inline void FillSpan2(short x1,short x2,unsigned char* addr) {
    static const unsigned short table1[16] = {0xffff,0x7fff,0x3fff,0x1fff,
                                              0x0fff,0x07ff,0x03ff,0x01ff,
                                              0x00ff,0x007f,0x003f,0x001f,
                                              0x000f,0x0007,0x0003,0x0001};
    static const unsigned short table2[16] = {0x0000,0x8000,0xc000,0xe000,
                                              0xf000,0xf800,0xfc00,0xfe00,
                                              0xff00,0xff80,0xffc0,0xffe0,
                                              0xfff0,0xfff8,0xfffc,0xfffe};
    short           dx;
    short           sx;
    unsigned short* p;

    if (x1>x2) {
        ASM_EXCHANGE(x1,x2);
    }
     if (x1<0){
       x1=0;
       if (x2<0)
       return;}
    if (x2>159) {
        x2=159;
        if (x1>159)
        return;}

    p  = (unsigned short*)(addr+((x1>>3)&0x1e));
    dx = x2-x1+1;
    sx = x1 & 0x000f;

    if (dx<16) {
        unsigned long val = (ASM_SWAP(table2[dx])) >> sx;
        *(unsigned long*)p &= ~val;

        return;
    }

    if (sx) {

        *p++ &= ~table1[sx];

        dx -= (16-sx);
    }
    while (dx >= 16) {

        *p++ &= ~0xffff;

        dx-=16;
    }

    if (dx) *p &= ~table2[dx];

}




void tri(short x1,short y1,short x2,short y2,short x3,short y3,int mode) {
    short m1,m2,m3,y,xa,xb,dy;
    unsigned char* addr;

    // check for degenerated triangle
    if ((y1==y2 && y2==y3) || (x1==x2 && x2==x3)) return;

    // sort points by their y coordinate ...
    // point with highest y will be afterwards (x1/y1), point with lowest y will be x3/y3)

    if (y2>=y1) {
        ASM_EXCHANGE(x1,x2);
        ASM_EXCHANGE(y1,y2);
    }

    if (y3>=y1) {
        ASM_EXCHANGE(x1,x3);
        ASM_EXCHANGE(y1,y3);
    }

    if (y3>=y2) {
        ASM_EXCHANGE(x2,x3);
        ASM_EXCHANGE(y2,y3);
    }


    //calculate dx/dy for highest-to-lowest point
    dy = y1-y3;
    if (dy) m1 =((x1-x3)<<7)/dy;
    else    m1 = 999;

    //calculate dx/dy for middle-to-lowest point
    dy = y2-y3;
    if (dy) m2 = ((x2-x3)<<7)/dy;
    else    m2 = 999;

    //calculate dx/dy for highest-to-middle point
    dy = y1-y2;
    if (dy) m3 =((x1-x2)<<7)/dy;
    else    m3 = 999;

    // start point is the lowest one point (xcoordinates shifted up by 7 bits)
    xa=x3<<7;
    xb=xa;

    // if highest-and-middle lowest point share their xcoordinate xb is the xcoord of middle point
    if (m2==999) xb=x2<<7;
   
   
   if (mode){
    addr = Vscreen1+(y3<<5)-(y3<<1);
    for (y=y3;y<y2;y++,xa+=m1,xb+=m2,addr+=30) 
    {
   if (y>-1&&y<100) 	
    FillSpan(/*(d1+)*/(xa>>7),/*d2+*/(xb>>7),addr);
    }
      addr = Vscreen1+(y2<<5)-(y2<<1);
    for (y=y2;y<y1;y++,xa+=m1,xb+=m3,addr+=30) 
    {
   if (y>-1&&y<100) 	
    FillSpan(/*d1+*/(xa>>7),/*d2+*/(xb>>7),addr);
    }
     
}
else
{
    addr = Vscreen1+(y3<<5)-(y3<<1);
    for (y=y3;y<y2;y++,xa+=m1,xb+=m2,addr+=30) {
       if (y>-1&&y<100) 	
    FillSpan2(/*d1+*/(xa>>7),/*d2+*/(xb>>7),addr);
}    
    addr = Vscreen1+(y2<<5)-(y2<<1);
    for (y=y2;y<y1;y++,xa+=m1,xb+=m3,addr+=30) {
       if (y>-1&&y<100) 	
    FillSpan2(/*d1+*/(xa>>7),/*d2+*/(xb>>7),addr);
}

}

}
//End of triangle filling routine***********************************************


//Key Detection while drawing ellipse

int touche2(int *x,int *y,int xx,int yy)
{
	int s=ngetchx();
	
	switch(s)
	{
	case 264:
	return 1;
	case 13:
	return 13;
			case 16722:
		if (*x+xx!=0&&xx-*x!=115) *x-=1;
		break;
		case 16728:
		if (*x+xx!=115&&+xx-*x!=0) *x+=1;
    break;
    case 16721:
		if (*y+yy!=0&&yy-*y!=75) *y-=1;
		break;
		case 16724:
		if (*y+yy!=75&&yy-*y!=0) *y+=1;
		break;
		case 338:
		if (*x+xx>4&&xx-*x<111) *x-=5;
		break;
		case 344:
		if (*x+xx<111&&xx-*x>4) *x+=5;
		break;
		case 337:
		if (*y+yy>4&&yy-*y<71) *y-=5;
		break;
		case 340:
		if (*y+yy<71&&yy-*y>4) *y+=5;
break;
}
	return 0;
}

//Key Detection
int touche(int *x,int *y,int xx,int yy)
{
	int s=ngetchx();
	
	switch(s)
	{
	case 264:
	return 1;
	case 13:
	return 13;
		
		case 16722:
		if (*x+xx!=0) *x-=1;
		break;
		case 16728:
		if (*x+xx!=115) *x+=1;
    break;
    case 16721:
		if (*y+yy!=0) *y-=1;
		break;
		case 16724:
		if (*y+yy!=75) *y+=1;
		break;
		case 338:
		if (*x+xx>4) *x-=5;
		break;
		case 344:
		if (*x+xx<111) *x+=5;
		break;
		case 337:
		if (*y+yy>4) *y-=5;
		break;
		case 340:
		if (*y+yy<71) *y+=5;
break;
}
	return 0;
}



char *IntToStr (unsigned long an_integer)
{
  static char result [] = "   \0";    
	unsigned long x=an_integer;
  char *ptr = result + 3;
  while (an_integer)
    {
      *ptr-- = an_integer % 10 + '0';
      an_integer/=10;
    }
    if (x<100) *ptr-- = '0';
    if (x<10) *ptr-- = '0';
    if (x==0) *ptr-- = '0';
     
      return ptr;
}










